/* SpotWelder Steuerung
 * Harald Sattler, 2019-06-29
 *
 * Der Attiny85 im DigiSpark laeuft mit 16,5 Mhz
 *
 * Basisplatine mit Anschluessen fuer 
 * - Fusspedal zum Ausloesen des Schweissimpulses
 * - Spannungsversorgung (DC, max. 14V. Der Spanungsregler des DigiSpark versorgt auch den Rest der Schaltung)
 * - SolidState Relais das den Mikrowellen-Trafo schaltet
 * - Stecksockel fuer den DigiSpark MicroController
 * - Stecksockel fuer das OLED Display (1,6" Diagonale, 128 x 64 Pixel)
 *
 * Mit einem Spindelpoti wird die Impulsdauer eingestellt
 * Ein Microtaster parallel zum Anschluss fuer den Fussschalter loest den Impuls ebenfalls aus (Test)
 * Der Impuls wird nur einmal ausgeloest, der Taster/Fussschalter muss losgelassen werden, bevor erneut ausgeloest werden kann.
 *
 * Der Taster/Fusspedal bentigt einen externen PullUp mit maximal 1,5 kOhm, da am selben Pin die Beschaltung fr den USB haengt.
 *   Die Z-Diode zieht ueber einen 68 Ohm Widerstand nach Masse. Mit PullUp von 1,5 kOhm erreicht der Pin einen Ruhepegel von
 *   ~3,2 V, was als High erkannt wird.
 * 
 * Mit dem Poti wird die Impulsdauer eingestellt und auf dem OLED-Display angezeigt.
 * Mit dem Taster wird der Impuls ausgelst.
 * Am Ausgang "Relais" wird ein SolidState Relais angesteuert, das den umgebauten
 *   Mikrowellen-Trafo primaerseitig fuer die Dauer des Impulses einschaltet.
 * 
 * Das Display zeigt die minimal und maximal einstellbare die aktuell eingestellte Impulsdauer an
 * Fuer die Dauer des Impulses wird - als visuelle Rueckmeldung - das Display invers geschaltet (bei (zu) kurzer Impulsdauer wird 
 *   nur ein schmaler Balken invers dargestellt)
 */
 
 // includes
 #include <Wire.h>
 
 // defines fuer die Pins
 #define TASTER 4
 #define POTI 	3
 #define RELAIS 1
 #define LED 5											// nicht verwenden, der DigiSpark wird angehalten, wenn an Pin 5 eine LED (oder ein PullUp!) angeschlossen ist
 
 // definitions IIC & OLED
#define PIN_I2C_SDA 0									// TX-Pin - die Pins koennen nicht geaendert werden!! Ist in der Lib so festgelegt
#define PIN_I2C_SCL 2									// RX-Pin - die Pins koennen nicht geaendert werden!! Ist in der Lib so festgelegt
#define OLED_address  0x3c							    // I2C-Adresse des OLED Display

// globale Variablen
 unsigned int potiWert;									// per analogRead() gelesener Wert des Poti
 unsigned long weldingTime;								// Wert des Impulses in [ms]
 unsigned long lastWeldingTime = 0;						// Merker fuer ScreenUpdate
 unsigned long weldingMicros;
 bool tasterSet = false;								// Flag fuer "ist der Taster gedrueckt?"
 
 // Puffer fuer OLED String-Ausgabe
char OLEDBuffer[34];
static char ZeilenOffset = 2;							// linker Rand fuer Textausgaben auf OLED

const unsigned int minImpulsAllowed = 10;				// minimale Impulsdauer
const unsigned int maxImpulsAllowed = 300;				// maximale Impulsdauer

void readPoti()
{
	// Poti auslesen, ergibt Wert 0..1023
	potiWert = analogRead(POTI);

	// daraus Zeit in ms in weldingTime ablegen
	weldingTime = map(potiWert, 0, 1023, minImpulsAllowed, maxImpulsAllowed);	// eingrenzen auf erlaubte min/max Werte
	weldingMicros = weldingTime * 1000;					// Probleme zur  Laufzeit mit delay(), daher delayMicroseconds() verwendet
}

void setup()
{
	// Pins initialisieren
	pinMode(TASTER, INPUT_PULLUP);
	pinMode(RELAIS, OUTPUT);
	pinMode(LED, INPUT);

	// nicht verwenden! (siehe oben)
	//for (unsigned char i = 0; i < 1; i++) { digitalWrite(RELAIS, 1); delay(500); digitalWrite(RELAIS, 0); delay(500);}
	// die OnBoard LED auch nicht, sonst wird der Trafo fr die Dauer des Blinkens eingschaltet (der zieht Einiges an Strom)
	//for (unsigned char i = 0; i < 10; i++) { digitalWrite(RELAIS, 1); delayMicroseconds(1000 * 1000); digitalWrite(RELAIS, 0); delayMicroseconds(1000 * 1000);}

	// Initialisierung IIC OLED Dispaly	
	//Wire.pins(PIN_I2C_SDA, PIN_I2C_SCL);
	Wire.begin();

	// Serial.println("Initialise OLED screen...");
	StartUp_OLED();
	clear_display();
	sendStrXY("SpotWelder", 0, ZeilenOffset);			// 16 chars / line
	sendStrXY("DigiSpark + OLED", 2, 0);
	// sendStrXY("  ", 4, ZeilenOffset);
	sendStrXY("Impuls:    ms", 6, ZeilenOffset);
	sendStrXY("xx", 6, ZeilenOffset + 8);
	delay(2000);
	clear_display();
}

void loop()
{
	static unsigned long looptime = 0;
	
	// Ausgabe auf OLED, jede Sekunde aktualisiert
	if (millis() > looptime + 500) 							// jede halbe Sekunde aktualisieren
	{	
		readPoti();											// liest Poti und liefert Impulsdauer in ms
		
		if (lastWeldingTime != weldingTime)					// hat sich der Wert geaendert? (Poti gedreht, Drift, ...)
		{
			// geanderte Impulszeit merken
			lastWeldingTime = weldingTime; 

			// dann Display aktualisieren
			sendStrXY("SpotWelder", 0, ZeilenOffset);		// 16 chars / line

			// Impulsdauer-Grenzen ausgeben
			sendStrXY("Dauer   ..    ms", 4, 0);
			dtostrf(minImpulsAllowed, 2, 0, OLEDBuffer);	// minimale Impulsdauer in String wandeln
			sendStrXY(OLEDBuffer, 4, 6);
			if (maxImpulsAllowed < 100) {
				dtostrf(maxImpulsAllowed, 2, 0, OLEDBuffer);// maximale Impulsdauer in String wandeln
			}
			else {
				dtostrf(maxImpulsAllowed, 3, 0, OLEDBuffer);// maximale Impulsdauer in String wandeln
			}
			sendStrXY(OLEDBuffer, 4, 10);

			// Temperatur als Int in den OLEDBuffer kopieren
			if (weldingTime < 100) {
				dtostrf(weldingTime, 2, 0, OLEDBuffer);		// aktuelle Impulsdauer in String wandeln
			}
			else {
				dtostrf(weldingTime, 3, 0, OLEDBuffer);		// aktuelle Impulsdauer in String wandeln
			}
			sendStrXY("Impuls:    ms", 6, ZeilenOffset);
			sendStrXY(OLEDBuffer, 6, ZeilenOffset + 8);
		}
		looptime = millis();
	}
	
	// Taster abfragen und Impuls ausloesen (Taster ist aktiv LOW)
	if((digitalRead(TASTER) == 0) && (tasterSet == false))	// Taster (nach offen) gedrueckt
	{
		// Set RelaisPin 
		sendcommand(0xa7);									// Set Inverse Display (visuelle Rueckmeldung)

		digitalWrite(RELAIS, 1);							// Impuls einschalten
		// delayMicroseconds(weldingMicros);					// Impulsdauer abwarten
		delay(weldingTime);								// Impulsdauer abwarten
		digitalWrite(RELAIS, 0);							// Impuls wieder ausschalten

		sendcommand(0xa6);									// Set Normal Display (not inverse) (visuelle Rueckmeldung)
		delay(200);											// kurz warten bevor ein neuer Impuls ausgeloest werden kann (Tastenprellen)

		tasterSet = true;
	}
	else if((tasterSet == true) && (digitalRead(TASTER) == 1))	// Taster wieder losgelassen
	{
		tasterSet = false;
	}
	else if ((digitalRead(TASTER) == 0) && (tasterSet == true))	// Taster festgehalten
	{
		delay(100);											// bei gedrueckter Taste Prellen unterdruecken
	}
}